import json
import requests

JIRA_BASE_ISSUE_URL = "https://motocommerce.atlassian.net/rest/api/2/issue"


def get_jira_ticket_key_from_branch(branch):
    """Construct a Jira ticket key based on the git branch name.

    For now it only replaces underscores with dashes and makes it uppercase.
    Eg. "proj_xxxx" -> "PROJ-XXXX"
    """
    if branch:
        jira_ticket_key = branch.upper().replace('_', '-')
    else:
        jira_ticket_key = None
    return jira_ticket_key


def get_pr_title_from_jira_ticket(branch, jira_email, jira_api_key):
    """Create a pull request title.

    Returns "jira ticket key: summary" as the title. Summary is pulled from the
    Jira ticket (summary field) and Jira ticket is chosen based on the branch.
    """
    jira_ticket_key = get_jira_ticket_key_from_branch(branch)
    if jira_ticket_key is None:
        return ""
    url = "{jira_issue_base_url}/{jira_ticket_key}?fields=summary".format(
        jira_issue_base_url=JIRA_BASE_ISSUE_URL,
        jira_ticket_key=jira_ticket_key)
    response = requests.get(
        url, auth=requests.auth.HTTPBasicAuth(jira_email, jira_api_key),
        headers= {"Content-Type" : "application/json"})
    if response.ok:
        response_data = json.loads(response.text)
        summary = response_data.get("fields", {}).get("summary", "")
        title = "{jira_ticket_key}: {summary}".format(
            jira_ticket_key=jira_ticket_key, summary=summary)
    else:
        title = jira_ticket_key
    return title



from github import Github, GithubException


class PullRequest():
    """PullRequest"""

    def __init__(self, repository_name, title, description, base, branch,
                 team_reviewers=None):
        self.repository_name = repository_name
        self.title = title
        self.description = description
        self.base = base
        self.branch = branch
        self.team_reviewers = team_reviewers

    def get_github_repository(self, github_token):
        """Fetch and return Github `Repository`."""
        try:
            gh = Github(github_token)
            gh_repository = gh.get_repo(self.repository_name)
        except GithubException:
            gh_repository = None
        return gh_repository

    def create(self, github_token):
        """Create Github `PullRequest` and set team reviewers.

        In case pull request already exists, update title and description.
        """
        gh_repository = self.get_github_repository(github_token)
        if gh_repository is None:
            return

        try:
            gh_pull_request = gh_repository.create_pull(
                title=self.title, body=self.description, base=self.base,
                head=self.branch)
            self.set_team_reviewers(gh_pull_request)
            html_url = gh_pull_request.html_url
            print("Pull request created at {}".format(html_url))
        except GithubException:
            gh_pull_request = self.update(github_token, gh_repository)
            if gh_pull_request:
                html_url = gh_pull_request.html_url
                print("Pull request updated at {}".format(html_url))
        return gh_pull_request

    def update(self, github_token, gh_repository):
        """Update Github `PullRequest` - set title and description.

        Find pull request by repository and branch.
        """
        assert(gh_repository is not None)

        gh_pull_requests = gh_repository.get_pulls(
            head="{repo}:{branch}".format(
                repo=self.repository_name, branch=self.branch),
            state="open")
        try:
            gh_pull_request = gh_pull_requests[0]
            gh_pull_request.edit(body=self.description, title=self.title)
        except IndexError:
            gh_pull_request = None
        return gh_pull_request

    def set_team_reviewers(self, github_pull_request):
        """Set team reviewers for the given pull request."""
        if self.team_reviewers:
            github_pull_request.create_review_request(
                team_reviewers=[self.team_reviewers])



import os
import sys

from pygit2 import GitError, Repository

from .config import load_config
from .jira_api import get_pr_title_from_jira_ticket
from .pull_request import PullRequest


def get_branch(repo_local_path=None):
    """Return current branch in a local repository `repo_local_path`."""
    if repo_local_path is None:
        repo_local_path = "."
    try:
        branch = Repository(repo_local_path).head.shorthand
    except GitError:
        branch = None
    return branch


def read_file(path):
    """Read file contents and return it as a string."""
    if path and os.path.exists(path):
        with open(path, "r") as file_handle:
            data = file_handle.read()
    else:
        data = ""
    return data


def main():
    """Create pull request on Github."""
    config = load_config()

    # Get git branch from config or look for the locally used one in repo-path.
    branch = config.get("branch") or get_branch(config.get("repo-path"))
    if branch is None:
        print("Invalid branch.")
        sys.exit(1)

    # Get pull request title from config or create one from the Jira ticket.
    title = config.get("pr-title")
    if title is None:
        title = get_pr_title_from_jira_ticket(
            branch=branch,
            jira_email=config.get("jira-email"),
            jira_api_key=config.get("jira-api-token"))

    # Read pull request description from file (path is set in config).
    description = read_file(path=config.get("pr-desc"))

    # Create pull request.
    pull_request = PullRequest(
        repository_name=config.get("repo"),
        title=title,
        description=description,
        base=config.get("pr-base", "master"),
        branch=branch,
        team_reviewers=config.get("pr-team"))

    pull_request.create(github_token=config.get("github-token"))


if __name__ == "__main__":
    main()



import argparse
try:
    import configparser
except ImportError:
    import ConfigParser as configparser
import os
import sys


CONFIG_FILE_NAME = "lazy_pr.ini"
ENV_PREFIX = "LAZY_PR_"


def get_command_line_options():
    """Parse and return command line arguments."""
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', '--config-file', help='config file path')
    parser.add_argument('-jt', '--jira-api-token', help='Jira API Token')
    parser.add_argument('-je', '--jira-email', help='Jira Email')
    parser.add_argument('-gt', '--github-token', help='Github Token')
    parser.add_argument('-r', '--repo', help='Git Repository')
    parser.add_argument('-rp', '--repo-path', help='Git Repository Local Path')
    parser.add_argument('-b', '--branch', help='Git Branch')
    parser.add_argument('-s', '--pr-base', help='PR Base')
    parser.add_argument('-t', '--pr-title', help='PR Title')
    parser.add_argument('-d', '--pr-desc', help='PR Description or Path')
    parser.add_argument('-tm', '--pr-team', help='PR Team Reviewers.')

    return parser.parse_args()


def get_config_file_options(config_file, section_name="default"):
    """Read config file and return `section_name` values."""
    config = configparser.SafeConfigParser()
    config.read([config_file])
    return dict(config.items(section_name))


def get_env_options(prefix):
    """Read and return environment variables prefixed with `prefix`.

    Prefix is stripped, names are lower cased and underscores are replaced with
    dashes.
    """
    options = {}
    for name in os.environ:
        if name.startswith(prefix):
            target_name = name[len(prefix):].lower().replace("_", "-")
            options[target_name] = os.environ[name]
    return options


def load_config():
    """Read and return configuration options."""
    cli_options = get_command_line_options()

    # Read configuration from .ini file.
    config_file = cli_options.config_file
    if config_file is None:
        # Read configuration from ~/.config/lazy_pr.ini.
        home = os.path.expanduser("~")
        config_file = os.path.join(home, '.config', CONFIG_FILE_NAME)
    config_options = get_config_file_options(config_file)

    # Override config with environment variables.
    env_options = get_env_options(ENV_PREFIX)
    for option_name in env_options:
        config_options[option_name] = env_options[option_name]

    # Override config with command line arguments.
    for option_key in vars(cli_options):
        option_value = getattr(cli_options, option_key)
        if option_value is not None:
            option_name = option_key.replace("_", "-")
            config_options[option_name] = option_value

    return config_options



from setuptools import find_packages, setup

setup(
    name="lazypr",
    version="0.1",
    description="Creating a pull request on GitHub",
    author="Sanja Segan",
    author_email="",
    url="https://github.com/sanjaz/lazypr",
    packages=find_packages(),
    include_package_data=True,
    test_suite="tests",
    install_requires=["PyGithub", "pygit2"],
    extras_require=None,
    tests_require=["mock", "pytest<5.0"],
    entry_points={"console_scripts": ["lazypr = lazypr.main:main"]},
)
